/*
 Function PROB = genlikE_shrink_color(DETAIL,DETAIL_CH2, DETAIL_CH3, LIKELIHOOD_RATIO, PRIOR_RATIO, WINDOW_HALFSIZE, PRIOR_FACTOR)   
 calculates the shrinage factor for each wavelet coefficient in a given subband, where
 this shrinkage factor is the probability that a coefficient contains a significant 
 noise-free component (i.e., signal of interest).

 
 This is an extension of the greyscale GENLIK denoising method described in:
 A. Pizurica, Image Denoising Using Wavelets and Spatial Context Modeling, 
 Ph.D. thesis, Ghent University, June 2002.


 Input:

    DETAIL - noisy subband
    DETAIL_CH2 and DETAIL_CH3 - corresponding subbands from other two color channels
    LIKELIHOOD_RATIO - look-up table 1: ratio of the conditional densities of coefficient 
                       magnitudes. This is a monotonic increasing exponentially shaped function 
					   of the coefficient magnitude.
    PRIOR_RATIO - look-up table 2: ratio of the conditional densities of locally averaged coefficient 
                  magnitudes. This is a monotonic increasing exponentially shaped function 
			      of the locally averaged magnitude.
    WINDOW HALF SIZE - e.g., equals 1 for 3x3 window and equals 2 for 5x5 window 
    PRIOR FACTOR - ratio of global probabilities of signal presence and absence in a given subband
   
   
 Output:

    PROB - array of estimated probabilities for each coefficient from the input DETAIL array

 Author: Aleksandra Pizurica/TELIN/Ghent University
 E-mail: Aleksandra.Pizurica@telin.UGent.be
 Last modified: February 2004.
 Copyright: All software, documentation, and related files in this distribution
            are Copyright (c) 2003 Aleksandra Pizurica. All rights reserved.

 This software is distributed and licensed to you on a non-exclusive 
 basis, free-of-charge. Redistribution and use in source and binary forms, 
 with or without modification, are permitted provided that the above 
 copyright notice is clearly maintained.
 */




#include "mex.h"
#include <math.h>


double absol(double a)
{
 double b;
 if(a<0)
   b=-a;
else
   b=a;
return b;
}


void workFcn(double **X, double **Y1, double **Y2, double *P, int M, int N, 
             double *Likelihood_ratio, double *Prior_ratio, int len_l, int len_p, double r, int W)
{ 
int i,j,k,u,v,L,m,H1,H2,V1,V2,e;
double Energy,Eta,Ksi,Count;
double *Measure;

L=M*N;
Measure=malloc(L*sizeof(double)); 

for(i=0; i<M; i++)
 for(j=0; j<N; j++)
   Measure[i+j*M] = abs(X[i][j]);


 for(i=0; i<M; i++){
	for(j=0; j<N; j++){
      
      k=i+j*M;
      m=Measure[k];
      if (m>len_l-1)
        m=len_l-1;
      Ksi=Likelihood_ratio[m];

   
      /* Determine the local window*/      
      if(i>=W)
         H1=W;
      else
         H1=i;

      if(i<M-W)
         H2=W;
      else
         H2=M-i-1;

      if(j>=W)
         V1=W;
      else
         V1=j;

      if(j<N-W)
         V2=W;
      else
         V2=N-j-1;
     /* The local window specified*/      

     Energy=0;
     Count=0;

      for(u=i-H1; u<i+H2+1; u++){
	      for(v=j-V1; v<j+V2+1; v++){
                Energy=Energy+absol(X[u][v])+absol(Y1[u][v])+absol(Y2[u][v]);
                Count+=1;
         }
      }

      Energy=(Energy-absol(X[i][j]))/(3*Count-1);
      e=Energy;
      if (e>len_p-1)
        e=len_p-1;
      Eta=r*Prior_ratio[e];

	   P[i+j*M]=Ksi*Eta/(1+Ksi*Eta);
     	  
	}
}

free(Measure);
             
}  /* End of workFcn */




/*Defining the gateway function, i.e. the mex function*/

#define DETAIL  prhs[0]
#define DETAIL_CHANNEL2  prhs[1]
#define DETAIL_CHANNEL3  prhs[2]
#define LIKELIHOOD_RATIO  prhs[3]
#define PRIOR_RATIO  prhs[4]
#define WINDOW_HALFSIZE  prhs[5]
#define PRIOR_FACTOR  prhs[6]

#define PROB plhs[0]



 
void mexFunction(
               int nlhs,
               mxArray  *plhs[],
               int     nrhs,
               const mxArray  *prhs[]
               )
{
  int row, col, i, M, N, ITERA, len_l, len_p, W;
  double Prior_factor;
  double **Detail, **Detail_ch2,**Detail_ch3, *Prob, *Likelihood_ratio, *Prior_ratio;

  if (nlhs!=1)
      mexErrMsgTxt("genlikE_shrink requires one output arguments [PROB].");
  if (nrhs!=7)
      mexErrMsgTxt("genlikE_shrink_color requires seven input arguments [DETAIL DETAIL_CHANNEL2 DETAIL_CHANNEL3 LIKELIHOOD_RATIO PRIOR_RATIO WINDOW_HALFSIZE PRIOR_FACTOR]");

  M=mxGetM(DETAIL);
  N=mxGetN(DETAIL);
  len_l=mxGetN(LIKELIHOOD_RATIO);
  len_p=mxGetN(PRIOR_RATIO);

  W=mxGetScalar(WINDOW_HALFSIZE);
  Prior_factor=mxGetScalar(PRIOR_FACTOR);
    
  /* Allocate memory for return matrices */
  PROB = mxCreateDoubleMatrix(M, N, mxREAL);  
  Prob = mxGetPr(PROB);

  /* Dynamic allocation of memory for input arrays*/

  Detail = malloc(M*sizeof(int));
  Detail_ch2 = malloc(M*sizeof(int));
  Detail_ch3 = malloc(M*sizeof(int));
  for(i=0;i<M;i++)
  {
    Detail[i]=malloc(N*sizeof(double));
    Detail_ch2[i]=malloc(N*sizeof(double));
    Detail_ch3[i]=malloc(N*sizeof(double));
  }


  Likelihood_ratio = malloc(len_l*sizeof(double));
  Prior_ratio = malloc(len_p*sizeof(double));




  /* Convert input arrays to 2x2 C arrays (MATLAB stores a two-dimensional matrix 
     in memory as a one-dimensional array) */

         for (col=0; col < N; col++)
            for (row=0; row < M; row++)
	      {
               Detail[row][col] = (mxGetPr(DETAIL))[row+col*M];
			   Detail_ch2[row][col] = (mxGetPr(DETAIL_CHANNEL2))[row+col*M];
			   Detail_ch3[row][col] = (mxGetPr(DETAIL_CHANNEL3))[row+col*M];
	      }

         for (col=0; col < len_l; col++)
               Likelihood_ratio[col] = (mxGetPr(LIKELIHOOD_RATIO))[col];

         for (col=0; col < len_p; col++)
               Prior_ratio[col] = (mxGetPr(PRIOR_RATIO))[col];
    

/*Call workFcn function*/ 
workFcn(Detail,Detail_ch2,Detail_ch3,Prob,M,N,Likelihood_ratio,Prior_ratio,len_l,len_p,Prior_factor,W);

for(i=0;i<M;i++)
{
 free(Detail[i]);
 free(Detail_ch2[i]);
 free(Detail_ch3[i]); 
}
free(Detail);
free(Detail_ch2);
free(Detail_ch3);

free(Likelihood_ratio);
free(Prior_ratio);
 
}/*mexFcn*/







