
% Function X=genlikSI_ColorFun(Xnoisy,NoiseStd,W,h,SigIntDEF)
% applies wavelet domain image denoisinig using generalized likelihood ratio. 
% 
% This is an extension of the greyscale GENLIK algorithm with the shift invariant (SI) transform and 
% using the generalized Laplacian prior, described in: 
% A. Pizurica, "Image Denoising Using Wavelets and Spatial Context Modeling",

% PhD. thesis, Ghent UNiversity, June 2002.
%
% Version for COLOR images
%
% INPUT:
%   Xnoisy - noisy image
%   NoiseStd - noise standard deviation
%   W - "half window size", e.g., W=1 for 3x3 window and W=2 for 5x5 window 
%   h - wavelet filter
%   SigIntDEF - A multiplication factor for adjusting the notion of the "signal of interest" 
%               Threshold that defines the signal of interest equals the noise standard deviation. 
%               This threshold yields the results that minimize the mean squred error. In some cases 
%               smaller threshold value may yield visually better result (more texture is preserved at 
%               the expense of the excessive noise). A user can define other threshold by choosing 
%               ``other signal of interest'' and specifying the value of the multiplication factor  
%               SigIntDEF for the threshold that define the signal of interest.
%
% OUTPUT:
%    X - denoised image
%
% 
% Author: Aleksandra Pizurica/TELIN/Ghent University
% E-mail: Aleksandra.Pizurica@telin.UGent.be
% Last modified: June 2003.
% Copyright: All software, documentation, and related files in this distribution
%            are Copyright (c) 2003 Aleksandra Pizurica. All rights reserved.
%
% This denoising routine uses wavelet decomposition and reconstruction functions
% mrdwt.c and mirdwt.c developed by Rice University, Houston, Texas and its contributors.
%
% This software is distributed and licensed to you on a non-exclusive 
% basis, free-of-charge. Redistribution and use in source and binary forms, 
% with or without modification, are permitted provided that the above 
% copyright notice is clearly maintained.


function X=genlikSI_ColorFun(Xnoisy,NoiseStd,W,h,SigIntDEF);

[M,N]=size(Xnoisy(:,:,1)); 


%=====================================================  
 % Decomposition 
  
 L=4;
 [yl1,yh1,L] = mrdwt(Xnoisy(:,:,1)/4,h,L);
 % (input image is here rescaled to make faster the subsequent convolutions 
 % that apear in functions MagPDFs_varT.m and AverageMagPDFs.m) 
 
 LH11 = yh1(:,1:N); 
 HL11 = yh1(:,N+1:2*N); 
 HH11 = yh1(:,2*N+1:3*N);
 
 LH21 = yh1(:,3*N+1:4*N); 
 HL21 = yh1(:,4*N+1:5*N); 
 HH21 = yh1(:,5*N+1:6*N);
 
 LH31 = yh1(:,6*N+1:7*N); 
 HL31 = yh1(:,7*N+1:8*N); 
 HH31 = yh1(:,8*N+1:9*N);
 
 LH41 = yh1(:,9*N+1:10*N); 
 HL41 = yh1(:,10*N+1:11*N); 
 HH41 = yh1(:,11*N+1:12*N);
 
  
 [yl2,yh2,L] = mrdwt(Xnoisy(:,:,2)/4,h,L);
 % (input image is here rescaled to make faster the subsequent convolutions 
 % that apear in functions MagPDFs_varT.m and AverageMagPDFs.m) 
 
 LH12 = yh2(:,1:N); 
 HL12 = yh2(:,N+1:2*N); 
 HH12 = yh2(:,2*N+1:3*N);
 
 LH22 = yh2(:,3*N+1:4*N); 
 HL22 = yh2(:,4*N+1:5*N); 
 HH22 = yh2(:,5*N+1:6*N);
 
 LH32 = yh2(:,6*N+1:7*N); 
 HL32 = yh2(:,7*N+1:8*N); 
 HH32 = yh2(:,8*N+1:9*N);
 
 LH42 = yh2(:,9*N+1:10*N); 
 HL42 = yh2(:,10*N+1:11*N); 
 HH42 = yh2(:,11*N+1:12*N);
 
 
 
 [yl3,yh3,L] = mrdwt(Xnoisy(:,:,3)/4,h,L);
 % (input image is here rescaled to make faster the subsequent convolutions 
 % that apear in functions MagPDFs_varT.m and AverageMagPDFs.m) 
 
 LH13 = yh3(:,1:N); 
 HL13 = yh3(:,N+1:2*N); 
 HH13 = yh3(:,2*N+1:3*N);
 
 LH23 = yh3(:,3*N+1:4*N); 
 HL23 = yh3(:,4*N+1:5*N); 
 HH23 = yh3(:,5*N+1:6*N);
 
 LH33 = yh3(:,6*N+1:7*N); 
 HL33= yh3(:,7*N+1:8*N); 
 HH33 = yh3(:,8*N+1:9*N);
 
 LH43 = yh3(:,9*N+1:10*N); 
 HL43 = yh3(:,10*N+1:11*N); 
 HH43 = yh3(:,11*N+1:12*N);
 
 
 
 
% Due to rescaling of gray values

sigma=NoiseStd/4;
T=SigIntDEF*sigma;


% Processing scale 2^4:
Scale=4
[HL41,HL42,HL43]=rem_noise_GenLap_color(HL41,HL42,HL43,W,sigma,T);
Denoised_Bands='3/36'
[LH41,LH42,LH43]=rem_noise_GenLap_color(LH41,LH42,LH43,W,sigma,T);
Denoised_Bands='6/36'
[HH41,HH42,HH43]=rem_noise_GenLap_color(HH41,HH42,HH43,W,sigma,T);
Denoised_Bands='9/36'

 yh1(:,9*N+1:10*N)=LH41; 
 yh1(:,10*N+1:11*N)=HL41; 
 yh1(:,11*N+1:12*N)=HH41;
 
 clear LH41 HL41 HL41;
 
 yh2(:,9*N+1:10*N)=LH42; 
 yh2(:,10*N+1:11*N)=HL42; 
 yh2(:,11*N+1:12*N)=HH42;
 
 clear LH42 HL42 HL42;
 
 yh3(:,9*N+1:10*N)=LH43; 
 yh3(:,10*N+1:11*N)=HL43; 
 yh3(:,11*N+1:12*N)=HH43;
 
 clear LH43 HL43 HL43;

% Processing scale 2^3:
Scale=3

[HL31,HL32,HL33]=rem_noise_GenLap_color(HL31,HL32,HL33,W,sigma,T);
Denoised_Bands='12/36'
[LH31,LH32,LH33]=rem_noise_GenLap_color(LH31,LH32,LH33,W,sigma,T);
Denoised_Bands='15/36'
[HH31,HH32,HH33]=rem_noise_GenLap_color(HH31,HH32,HH33,W,sigma,T);
Denoised_Bands='18/36'

 yh1(:,6*N+1:7*N)=LH31; 
 yh1(:,7*N+1:8*N)=HL31; 
 yh1(:,8*N+1:9*N)=HH31;
 
 clear LH31 HL31 HL31;
 
 yh2(:,6*N+1:7*N)=LH32; 
 yh2(:,7*N+1:8*N)=HL32; 
 yh2(:,8*N+1:9*N)=HH32;
 
 clear LH32 HL32 HL32;
 
 yh3(:,6*N+1:7*N)=LH33; 
 yh3(:,7*N+1:8*N)=HL33; 
 yh3(:,8*N+1:9*N)=HH33;
 
 clear LH33 HL33 HL33;

% Processing scale 2^2:
Scale=2

[HL21,HL22,HL23]=rem_noise_GenLap_color(HL21,HL22,HL23,W,sigma,T);
Denoised_Bands='21/36'
[LH21,LH22,LH23]=rem_noise_GenLap_color(LH21,LH22,LH23,W,sigma,T);
Denoised_Bands='24/36'
[HH21,HH22,HH23]=rem_noise_GenLap_color(HH21,HH22,HH23,W,sigma,T);
Denoised_Bands='27/36'

 yh1(:,3*N+1:4*N)=LH21; 
 yh1(:,4*N+1:5*N)=HL21; 
 yh1(:,5*N+1:6*N)=HH21;
 
 clear LH21 HL21 HL21;
 
 yh2(:,3*N+1:4*N)=LH22; 
 yh2(:,4*N+1:5*N)=HL22; 
 yh2(:,5*N+1:6*N)=HH22;
 
 clear LH22 HL22 HL22;
 
 yh3(:,3*N+1:4*N)=LH23; 
 yh3(:,4*N+1:5*N)=HL23; 
 yh3(:,5*N+1:6*N)=HH23;

 
 clear LH23 HL23 HL23;


% Processing scale 2^1:
Scale=1;

[HL11,HL12,HL13]=rem_noise_GenLap_color(HL11,HL12,HL13,W,sigma,T);
Denoised_Bands='30/36'
[LH11,LH12,LH13]=rem_noise_GenLap_color(LH11,LH12,LH13,W,sigma,T);
Denoised_Bands='33/36'
[HH11,HH12,HH13]=rem_noise_GenLap_color(HH11,HH12,HH13,W,sigma,T);
Denoised_Bands='36/36'

 yh1(:,1:N)=LH11; 
 yh1(:,N+1:2*N)=HL11; 
 yh1(:,2*N+1:3*N)=HH11;
 
 clear LH11 HL11 HL11;
 
 yh2(:,1:N)=LH12; 
 yh2(:,N+1:2*N)=HL12; 
 yh2(:,2*N+1:3*N)=HH12;
 
 clear LH12 HL12 HL12;
 
 yh3(:,1:N)=LH13; 
 yh3(:,N+1:2*N)=HL13; 
 yh3(:,2*N+1:3*N)=HH13;
 
 clear LH13 HL13 HL13;


%================================================
% Reconstruction

 
[Xden1,L] = mirdwt(yl1,yh1,h,L);
[Xden2,L] = mirdwt(yl2,yh2,h,L);
[Xden3,L] = mirdwt(yl3,yh3,h,L);

Xden1=4*Xden1;
Xden2=4*Xden2;
Xden3=4*Xden3;

X(:,:,1)=Xden1;
X(:,:,2)=Xden2;
X(:,:,3)=Xden3;
