% Function D_cl = rem_noise_ColorCompFull(D,Dch2,Dch3,W,sigma,T)
% shrinks all the coefficients in a given subband according to the generalized likelihood ratio
% and assuming the generalized Laplacian prior on noise-free coefficients.
% 
% This is an extension of the grescale GENLIK denoising method described in
% A. Pizurica, Image Denoising Using Wavelets and Spatial Context Modeling, Ph.D. thesis,
% Ghent University, June 2002.
%
% INPUT:
%   D - noisy subband
%   Dch2 and Dch3 - corresponding noisy subbands from other two color channels
%   W - "half window size", e.g., W=1 for 3x3 window and W=2 for 5x5 window 
%   sigma - noise standard deviation
%   T - threshold that defines the notion of the "signal of interest"
%
% OUTPUT:
%   D_cl - denoised ("cleaned") subband
%
% Author: Aleksandra Pizurica/TELIN/Ghent University
% E-mail: Aleksandra.Pizurica@telin.UGent.be
% Last modified: November 2003.
% Copyright: All software, documentation, and related files in this distribution
%            are Copyright (c) 2003 Aleksandra Pizurica. All rights reserved.
%
% This software is distributed and licensed to you on a non-exclusive 
% basis, free-of-charge. Redistribution and use in source and binary forms, 
% with or without modification, are permitted provided that the above 
% copyright notice is clearly maintained.


function D_cl = rem_noise_ColorCompFull(D,Dchannel2,Dchannel3,W,sigma,T);


[M,N]=size(D);

 % estimate parameters of the pdf of noise free wavelet coefficients
 % Laplacian model exp(-x/s)^p is assumed
 
 
 [p,s]=FindGenLapPar(D,sigma);
 [p2,s2]=FindGenLapPar(Dchannel2,sigma);
 [p3,s3]=FindGenLapPar(Dchannel3,sigma);

 
 I=gammainc((sigma/s)^p,1/p);
 
 PrFac=(1-I)/(I);
 if(PrFac<1e-6)
     PrFac=0;
 end
 
 if(PrFac==0)
    D_cl=zeros(M,N);
else
    Lim=ceil(max(max(abs(D))));

   % Compute pdfs p(m|0) and p(m|1) in the range m=0:Lim
   [LR,pdf_m_s,pdf_m_n]=MagPDFs_varT(p,s,sigma,T,Lim);
   [LR,pdf_m_s2,pdf_m_n2]=MagPDFs_varT(p2,s2,sigma,T,Lim);
   [LR,pdf_m_s3,pdf_m_n3]=MagPDFs_varT(p3,s3,sigma,T,Lim);
 

   % Compute pdfs p(e|0) and p(e|1) in the range m=0:Lim
   %[PR,pdf_am_s,pdf_am_n]=AverageMagPDFs(pdf_m_s,pdf_m_n,W,Lim);
   
         Lim=length(pdf_m_s)-1;

         if(sum(pdf_m_s)==0)  
             PR(1:Lim+1)=0;
             pdf_am_s=0;
             pdf_am_n=0;
         else
             pdf_am_s=conv(pdf_m_s,pdf_m_s2);
             pdf_am_s=2*pdf_am_s(1:2:length(pdf_am_s));
             pdf_am_s=pdf_am_s/sum(pdf_am_s);
             
             pdf_am_s=conv(pdf_am_s,pdf_m_s3);
             pdf_am_s=2*pdf_am_s(1:2:length(pdf_am_s));
             pdf_am_s=pdf_am_s/sum(pdf_am_s);
             
   
             pdf_am_n=conv(pdf_m_n,pdf_m_n2);
             pdf_am_n=2*pdf_am_n(1:2:length(pdf_am_n));
             pdf_am_n=pdf_am_n/sum(pdf_am_n);
             
             pdf_am_n=conv(pdf_am_n,pdf_m_n3);
             pdf_am_n=2*pdf_am_n(1:2:length(pdf_am_n));
             pdf_am_n=pdf_am_n/sum(pdf_am_n);
             
             
          
             k=1;
             while(pdf_am_n(k)==0)
                 k=k+1;
             end
             r(1:k)=0;

             pdf_am_n_c=1000*pdf_am_n;
             pdf_am_s_c=1000*pdf_am_s;

             % to prevent numerical errors
             [Max,ind]=max(pdf_am_n);

             for i=ind:Lim
                 if(pdf_am_n(i)<1e-10)
                    pdf_am_n_c(i+1:Lim+1)=pdf_am_n_c(i);
                    pdf_am_s_c(i+1:Lim+1)=pdf_am_s_c(i);
                 end
             end
             PR(k:(Lim+1))=pdf_am_s_c(k:(Lim+1))./pdf_am_n_c(k:(Lim+1));
          end

   

    SHRINK=genlikE_shrink_color(D,Dchannel2,Dchannel3,LR,PR,W,PrFac);
    D_cl=D.*SHRINK;

end







